
#region module logging

get-module hyper-v | select log*

(get-module hyper-V).LogPipelineExecutionDetails = $True
#can also be set via a GPO

Get-VM chi-dc01 

get-eventlog -LogName "Windows PowerShell" -newest 5 -Message "Pipeline*"

$filterXML = @'
<QueryList>
  <Query Id="0" Path="Microsoft-Windows-PowerShell/Operational">
    <Select Path="Microsoft-Windows-PowerShell/Operational">*[System[(Task=106) and (Opcode=20)]]</Select>
  </Query>
</QueryList>
'@

 Get-WinEvent -FilterXml $filterXML -MaxEvents 50 | where {$_.message -match  "CommandInvocation"}

#endregion

#region transcripts
#also works in v4 with KB3000850 but without new parameters
$computername = "chi-test01"

Enter-pssession -ComputerName $computername

mkdir c:\mylogs

cd \
help Start-Transcript
$t = Start-Transcript -OutputDirectory c:\mylogs -IncludeInvocationHeader
$t
$t.path
$PSVersionTable
#start another transcript for a script
$sb = {
    start-transcript "c:\mylogs\b.txt" -IncludeInvocationHeader | out-null
    get-ciminstance win32_computersystem -verbose
    get-ciminstance win32_operatingSystem | select Caption
    Stop-Transcript
}
&$sb
get-service m*
Stop-Transcript
cls
#"nested" transcript
get-content "c:\mylogs\b.txt"
cls
#session transcript
get-content $t.path

cls

<#
for automatic transcription, enable the 
Turn on PowerShell Transcription via GPO
 Windows Components -> Administrative Templates -> Windows PowerShell. 
#>

function Enable-PSTranscription {
    [CmdletBinding(SupportsShouldProcess)]
    param(
        $OutputDirectory,
        [Switch] $IncludeInvocationHeader
    )

    ## Ensure the base path exists
    $basePath = "HKLM:\Software\Policies\Microsoft\Windows\PowerShell\Transcription"
    if(-not (Test-Path $basePath))
    {
        $null = New-Item $basePath -Force
    }

    ## Enable transcription
    Set-ItemProperty $basePath -Name EnableTranscripting -Value 1

    ## Set the output directory
    if($PSCmdlet.MyInvocation.BoundParameters.ContainsKey("OutputDirectory"))
    {
        Set-ItemProperty $basePath -Name OutputDirectory -Value $OutputDirectory
    }

    ## Set the invocation header
    if($IncludeInvocationHeader)
    {
        Set-ItemProperty $basePath -Name IncludeInvocationHeader -Value 1
    }
}

function Disable-PSTranscription {
[cmdletbinding(SupportsShouldProcess)]
Param()
    Remove-Item HKLM:\Software\Policies\Microsoft\Windows\PowerShell\Transcription -Force -Recurse
}

#turn it on
Enable-PSTranscription -IncludeInvocationHeader -OutputDirectory c:\mylogs

Exit-PSSession

#test it
get-date

Enter-pssession -ComputerName $computername

$env:__PSTranscriptPath
dir $env:__PSTranscriptPath

#do some a few things 
mkdir c:\Surprise

dir c:\windows\*.exe | sort lastwritetime | export-clixml c:\surprise\x.xml

get-process | sort WS -Descending | Select -first 5
ipconfig /all


#also works for things like this
$myType = @"
using System;
using System.Management.Automation;

namespace Ignorant
{
    public class Program
    {
        public static void Main()
        {
            var processes = PowerShell.Create().AddCommand("Get-Process").AddParameter("Name", "s*").Invoke();
            Console.WriteLine("You have " + processes.Count +
                " processes that start with 's'");
        }
    }
}
"@

add-type -TypeDefinition $mytype
[Ignorant.Program]::main()

#don't try to view an active transcript
Exit-PSSession

#view results
invoke-item '\\$computername\c$\mylogs'
dir c:\mylogs

#Suggestion: setup central share for all transcripts

#endregion

#region scriptblock logging

<#
enable the Turn on PowerShell Script Block Logging feature via GPO
Windows Components -> Administrative Templates -> Windows PowerShell. 
Configuration settings: 
HKLM:\Software\Policies\Microsoft\Windows\PowerShell\ScriptBlockLogging

Logged to:
Microsoft-Windows-PowerShell/Operational log
#>

Enter-PSSession -ComputerName $computername
cd \

function Enable-PSScriptBlockLogging {
[cmdletbinding(SupportsShouldProcess)]
Param()

    $basePath = "HKLM:\Software\Policies\Microsoft\Windows\PowerShell\ScriptBlockLogging"

    if(-not (Test-Path $basePath))  {
        $null = New-Item $basePath -Force
    }
    
    Set-ItemProperty $basePath -Name EnableScriptBlockLogging -Value "1"
}

function Disable-PSScriptBlockLogging {
[cmdletbinding(SupportsShouldProcess)]
Param()
    Remove-Item HKLM:\Software\Policies\Microsoft\Windows\PowerShell\ScriptBlockLogging -Force -Recurse
}

function Enable-PSScriptBlockInvocationLogging {
[cmdletbinding(SupportsShouldProcess)]
Param()
    $basePath = "HKLM:\Software\Policies\Microsoft\Windows\PowerShell\ScriptBlockLogging"

    if(-not (Test-Path $basePath))
    {
        $null = New-Item $basePath -Force
    }

    Set-ItemProperty $basePath -Name EnableScriptBlockInvocationLogging -Value "1"
} 

Enable-PSScriptBlockLogging
Enable-PSScriptBlockInvocationLogging

$sb = {Param([string]$s) get-service -displayname "*$s*"}

&$sb "micro"
#might be a delay in writing to the log
Get-WinEvent "Microsoft-Windows-PowerShell/Operational" -MaxEvents 3 |
Select TimeCreated,ID,Message | format-table -wrap

Invoke-Command $sb -ArgumentList "win"

#endregion
